/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.beans.request;

import java.util.ArrayList;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.RequestScoped;
import javax.faces.context.FacesContext;

import kieker.analysis.model.analysisMetaModel.MIDependency;
import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.webgui.beans.application.DependenciesBean;
import kieker.webgui.beans.session.SelectedProjectBean;

/**
 * This bean contains the currently chosen dependencies. This has to be request scoped, because the bean fetches the currently selected project when created.
 * 
 * @author Nils Christian Ehmke
 * @version 1.0
 */
@ManagedBean
@RequestScoped
public class SelectedDependenciesBean {

	/**
	 * This list contains the currently selected dependencies.
	 */
	private List<MIDependency> dependencies;
	/**
	 * This field contains the corresponding project, which will be modified by this bean.
	 */
	private final MIProject project;

	/**
	 * Creates a new instance of this class.
	 */
	public SelectedDependenciesBean() {
		this.dependencies = new ArrayList<MIDependency>();

		/* Try to get the currently selected project. */
		final FacesContext context = FacesContext.getCurrentInstance();

		final SelectedProjectBean selProjBean = context.getApplication().evaluateExpressionGet(context, "#{selectedProjectBean}",
				SelectedProjectBean.class);
		final DependenciesBean depBean = context.getApplication().evaluateExpressionGet(context, "#{dependenciesBean}",
				DependenciesBean.class);

		if (selProjBean != null) {
			this.project = selProjBean.getSelectedProject();
		} else {
			this.project = null;
		}

		/*
		 * The adding of the already existing dependencies within the project is troublesome. As the names cannot be compared, we have to do this manually and
		 * use our "own" dependencies which we have to compare with the dependencies in the project.
		 */
		if (selProjBean != null && depBean != null) {
			final List<MIDependency> availableDependencies = depBean.getDependencies();

			for (final MIDependency dependency : this.project.getDependencies()) {
				for (final MIDependency actDependency : availableDependencies) {
					if (actDependency.getFilePath().equals(dependency.getFilePath())) {
						this.dependencies.add(actDependency);
						continue;
					}
				}
			}
		}
	}

	/**
	 * Delivers the currently selected dependencies.
	 * 
	 * @return The selected dependencies.
	 */
	public List<MIDependency> getDependencies() {
		return this.dependencies;
	}

	/**
	 * Sets the currently selected dependencies to a new value.
	 * 
	 * @param dependencies
	 *            The newly selected dependencies.
	 */
	public void setDependencies(final List<MIDependency> dependencies) {
		this.dependencies = dependencies;
	}

	/**
	 * This method should be called as an action. It clears the project's dependencies and adds all currently selected dependencies to this project.
	 */
	public void submit() {
		if (this.project != null) {
			this.project.getDependencies().clear();
			this.project.getDependencies().addAll(this.dependencies);
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO, "", "Dependencies saved."));
		} else {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO, "", "No project selected."));
		}
	}

}
