/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.beans.session;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Modifier;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.context.FacesContext;

import kieker.analysis.model.analysisMetaModel.MIDependency;
import kieker.analysis.model.analysisMetaModel.MIFilter;
import kieker.analysis.model.analysisMetaModel.MIInputPort;
import kieker.analysis.model.analysisMetaModel.MIOutputPort;
import kieker.analysis.model.analysisMetaModel.MIPlugin;
import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.analysis.model.analysisMetaModel.MIProperty;
import kieker.analysis.model.analysisMetaModel.impl.MAnalysisMetaModelFactory;
import kieker.analysis.plugin.AbstractPlugin;
import kieker.analysis.plugin.filter.AbstractFilterPlugin;
import kieker.analysis.plugin.reader.AbstractReaderPlugin;
import kieker.analysis.repository.AbstractRepository;
import kieker.common.configuration.Configuration;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.webgui.beans.application.ProjectsBean;
import kieker.webgui.common.Connection;
import kieker.webgui.common.FileManager;
import kieker.webgui.common.PluginClassLoader;
import kieker.webgui.common.PluginFinder;

import org.eclipse.emf.common.util.EList;
import org.primefaces.context.RequestContext;

/**
 * This session bean stores the currently selected main project of the user and provides different methods to access and manipulate the properties of this project.
 * It is possible that this bean accesses other beans directly.
 * 
 * @author Nils Christian Ehmke
 * @version 1.0
 */
@ManagedBean
@SessionScoped
public class SelectedMainProjectBean extends Observable implements Observer {
	/**
	 * The logger within this class.
	 */
	private static final Log LOG = LogFactory.getLog(SelectedMainProjectBean.class);
	/**
	 * The error message used if a plugin could not be instantiated for various reasons.
	 */
	private static final String ERR_MSG_LOAD_PLUGIN = "Could not instantiate plugin.";
	/**
	 * The error message used if dependency could not loaded due to an invalid URL.
	 */
	private static final String ERR_MSG_INVALID_URL = "Invalid URL for dependency.";
	/**
	 * This constant is used as the host for the dependencies.
	 */
	private static final String URL_LOCALHOST = "localhost";
	/**
	 * This constant is used as a protocol for the dependencies.
	 */
	private static final String URL_PROTOCOL_FILE = "file";

	/**
	 * This collection contains all targets within the current view which will be updated when the selection of the main project changes.
	 */
	private static final Collection<String> UPDATE_TARGETS = Collections.unmodifiableCollection(Arrays.asList(new String[] { "projectsForm", "toolpalette",
		"centerForm", }));

	/**
	 * The main project of the current user.
	 */
	private MIProject mainProject;
	/**
	 * This list contains the connections (between the filters) within the current main project.
	 */
	private List<Connection> connections;

	private final ProjectsBean observableBean;

	/**
	 * Creates a new instance of this class.
	 */
	public SelectedMainProjectBean() {
		// Make sure that the collection for the connection is not empty. This will make the iteration over the collection
		this.connections = new ArrayList<Connection>();

		// Try to get the projects bean.
		final FacesContext context = FacesContext.getCurrentInstance();
		this.observableBean = context.getApplication().evaluateExpressionGet(context, "#{projectsBean}", ProjectsBean.class);
	}

	/**
	 * Delivers the main project of the current user. It can be null.
	 * 
	 * @return The user's main project.
	 */
	public final MIProject getMainProject() {
		return this.mainProject;
	}

	/**
	 * Sets the main project of the current user.
	 * 
	 * @param mainProject
	 *            The new main project of the current user.
	 */
	public final void setMainProject(final MIProject mainProject) {
		// Overtake the main project and its connection.
		this.mainProject = mainProject;
		this.connections = this.getConnectionsFromProject();

		// In the case we haven't set the main project to null, we inform the user about the change.
		if (this.mainProject != null) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(FacesMessage.SEVERITY_INFO, "", "New main project: " + mainProject.getName()));
		}

		// Inform the observers
		this.setChanged();
		this.notifyObservers(new MsgMainProjectSelection(mainProject));

		// Update the corresponding components
		final RequestContext requestContext = RequestContext.getCurrentInstance();
		requestContext.addPartialUpdateTargets(SelectedMainProjectBean.UPDATE_TARGETS);
	}

	/**
	 * This method delivers the available reader-plugins for the current main project. The delivered plugins are never abstract.
	 * 
	 * @return A list with all readers.
	 */
	public final List<Class<AbstractReaderPlugin>> getAvailableReaders() {
		final List<Class<AbstractReaderPlugin>> list = new ArrayList<Class<AbstractReaderPlugin>>();

		// Make sure there is a main project.
		if (this.mainProject != null) {
			// Run through all libraries
			for (final MIDependency lib : this.mainProject.getDependencies()) {
				// Make sure the plugin class loader knows about the lib
				try {
					PluginClassLoader.getInstance().addURL(
							new URL(SelectedMainProjectBean.URL_PROTOCOL_FILE, SelectedMainProjectBean.URL_LOCALHOST, FileManager.getInstance().getFullPath(lib)));
				} catch (final MalformedURLException ex) {
					SelectedMainProjectBean.LOG.warn(SelectedMainProjectBean.ERR_MSG_INVALID_URL, ex);
				}
				// Now try to find the plugins within the library and extract the readers.
				try {
					final List<Class<AbstractPlugin>> plugins = PluginFinder.getAllPluginsWithinJar(new URL(SelectedMainProjectBean.URL_PROTOCOL_FILE,
							SelectedMainProjectBean.URL_LOCALHOST, FileManager.getInstance().getFullPath(lib)));
					for (final Class<?> plugin : plugins) {
						if (!Modifier.isAbstract(plugin.getModifiers()) && AbstractReaderPlugin.class.isAssignableFrom(plugin)) {
							list.add((Class<AbstractReaderPlugin>) plugin);
						}
					}
				} catch (final MalformedURLException ex) {
					SelectedMainProjectBean.LOG.warn(SelectedMainProjectBean.ERR_MSG_INVALID_URL, ex);
				}
			}
		}

		return list;
	}

	/**
	 * This method delivers the available filter-plugins for the current main project. The delivered plugins are never abstract.
	 * 
	 * @return A list with all filter.
	 */
	public final List<Class<AbstractFilterPlugin>> getAvailableFilters() {
		final List<Class<AbstractFilterPlugin>> list = new ArrayList<Class<AbstractFilterPlugin>>();

		if (this.mainProject != null) {
			for (final MIDependency lib : this.mainProject.getDependencies()) {
				try {
					PluginClassLoader.getInstance().addURL(
							new URL(SelectedMainProjectBean.URL_PROTOCOL_FILE, SelectedMainProjectBean.URL_LOCALHOST, FileManager.getInstance().getFullPath(lib)));
				} catch (final MalformedURLException ex) {
					SelectedMainProjectBean.LOG.warn(SelectedMainProjectBean.ERR_MSG_INVALID_URL, ex);
				}
				try {
					final List<Class<AbstractPlugin>> plugins = PluginFinder.getAllPluginsWithinJar(new URL(SelectedMainProjectBean.URL_PROTOCOL_FILE,
							SelectedMainProjectBean.URL_LOCALHOST, FileManager.getInstance().getFullPath(lib)));
					for (final Class<?> plugin : plugins) {
						if (!Modifier.isAbstract(plugin.getModifiers()) && AbstractFilterPlugin.class.isAssignableFrom(plugin)) {
							list.add((Class<AbstractFilterPlugin>) plugin);
						}
					}
				} catch (final MalformedURLException ex) {
					SelectedMainProjectBean.LOG.warn(SelectedMainProjectBean.ERR_MSG_INVALID_URL, ex);
				}
			}
		}

		return list;

	}

	/**
	 * This method delivers the available repositories for the current main project. The delivered repositories are never abstract.
	 * 
	 * @return A list with all repositories.
	 */
	public final List<Class<AbstractRepository>> getAvailableRepositories() {
		final List<Class<AbstractRepository>> list = new ArrayList<Class<AbstractRepository>>();

		if (this.mainProject != null) {
			for (final MIDependency lib : this.mainProject.getDependencies()) {
				try {
					PluginClassLoader.getInstance().addURL(
							new URL(SelectedMainProjectBean.URL_PROTOCOL_FILE, SelectedMainProjectBean.URL_LOCALHOST, FileManager.getInstance().getFullPath(lib)));
				} catch (final MalformedURLException ex) {
					SelectedMainProjectBean.LOG.warn(SelectedMainProjectBean.ERR_MSG_INVALID_URL, ex);
				}
				try {
					final List<Class<AbstractRepository>> repositories = PluginFinder.getAllRepositoriesWithinJar(new URL(SelectedMainProjectBean.URL_PROTOCOL_FILE,
							SelectedMainProjectBean.URL_LOCALHOST, FileManager.getInstance().getFullPath(lib)));
					for (final Class<AbstractRepository> repository : repositories) {
						if (!Modifier.isAbstract(repository.getModifiers())) {
							list.add(repository);
						}
					}
				} catch (final MalformedURLException ex) {
					SelectedMainProjectBean.LOG.warn(SelectedMainProjectBean.ERR_MSG_INVALID_URL, ex);
				}
			}
		}

		return list;

	}

	/**
	 * This method uses the actual plugin to create the model configuration instances and add them to the model plugin instance.
	 * 
	 * @param plugin
	 *            The plugin whose configuration will be used.
	 * @param mPlugin
	 *            The model plugin which will be modified.
	 */
	private static void addConfiguration(final AbstractPlugin plugin, final MIPlugin mPlugin) {
		// Get the current configuration and use it to initialize the model plugin.
		final Configuration configuration = plugin.getCurrentConfiguration();
		final MAnalysisMetaModelFactory factory = new MAnalysisMetaModelFactory();

		// Run through all entries.
		final Iterator<Map.Entry<Object, Object>> iterator = configuration.entrySet().iterator();
		while (iterator.hasNext()) {
			final Map.Entry<Object, Object> entry = iterator.next();
			// Create a property object for the current entry.
			final MIProperty property = factory.createProperty();
			property.setName(entry.getKey().toString());
			property.setValue(entry.getValue().toString());

			mPlugin.getProperties().add(property);
		}
	}

	/**
	 * This method uses the actual plugin to create the model port instances and add them to the model plugin instance.
	 * 
	 * @param plugin
	 *            The plugin whose ports will be used.
	 * @param mPlugin
	 *            The model plugin which will be modified.
	 */
	private static void addPorts(final AbstractPlugin plugin, final MIPlugin mPlugin) {
		// Get the port and use them to initialize the model plugin.
		final String[] inputPortNames = plugin.getAllInputPortNames();
		final String[] outputPortNames = plugin.getAllOutputPortNames();

		final MAnalysisMetaModelFactory factory = new MAnalysisMetaModelFactory();

		if (mPlugin instanceof MIFilter) {
			for (final String inputPortName : inputPortNames) {
				final MIInputPort mInputPort = factory.createInputPort();
				mInputPort.setName(inputPortName);
				mInputPort.setParent((MIFilter) mPlugin);
			}
		}

		for (final String outputPortName : outputPortNames) {
			final MIOutputPort mOutputPort = factory.createOutputPort();
			mOutputPort.setName(outputPortName);
			mOutputPort.setParent(mPlugin);
		}
	}

	/**
	 * This method can be used to add a model plugin instance to the current main project using the given class.
	 * 
	 * @param pluginClass
	 *            The class which will be instantiated.
	 */
	public final void addPlugin(final Class<AbstractPlugin> pluginClass) {
		final MAnalysisMetaModelFactory factory = new MAnalysisMetaModelFactory();

		try {
			final AbstractPlugin plugin = pluginClass.getConstructor(Configuration.class).newInstance(new Configuration());
			MIPlugin mPlugin;

			if (AbstractReaderPlugin.class.isAssignableFrom(pluginClass)) {
				mPlugin = factory.createReader();
			} else {
				mPlugin = factory.createFilter();
			}

			mPlugin.setClassname(pluginClass.getCanonicalName());
			mPlugin.setName(pluginClass.getSimpleName());
			SelectedMainProjectBean.addConfiguration(plugin, mPlugin);
			SelectedMainProjectBean.addPorts(plugin, mPlugin);

			this.mainProject.getPlugins().add(mPlugin);

		} catch (final InstantiationException ex) {
			SelectedMainProjectBean.LOG.warn(SelectedMainProjectBean.ERR_MSG_LOAD_PLUGIN, ex);
		} catch (final NoSuchMethodException ex) {
			SelectedMainProjectBean.LOG.warn(SelectedMainProjectBean.ERR_MSG_LOAD_PLUGIN, ex);
		} catch (final SecurityException ex) {
			SelectedMainProjectBean.LOG.warn(SelectedMainProjectBean.ERR_MSG_LOAD_PLUGIN, ex);
		} catch (final IllegalAccessException ex) {
			SelectedMainProjectBean.LOG.warn(SelectedMainProjectBean.ERR_MSG_LOAD_PLUGIN, ex);
		} catch (final IllegalArgumentException ex) {
			SelectedMainProjectBean.LOG.warn(SelectedMainProjectBean.ERR_MSG_LOAD_PLUGIN, ex);
		} catch (final InvocationTargetException ex) {
			SelectedMainProjectBean.LOG.warn(SelectedMainProjectBean.ERR_MSG_LOAD_PLUGIN, ex);
		}

	}

	/**
	 * This method removes a given model plugin instance from the current main project. If this is the currently selected plugin from the user, it will be deselected
	 * as well. This means that this method accesses the <code>SelectedPluginBean</code> directly.
	 * 
	 * @param plugin
	 *            The project to be removed.
	 */
	public final void removePlugin(final MIPlugin plugin) {
		// TODO Replace with an observer
		this.mainProject.getPlugins().remove(plugin);

		// Get the currently selected plugin bean.
		final SelectedPluginBean selectedPluginBean = (SelectedPluginBean) FacesContext.getCurrentInstance().getExternalContext()
				.getSessionMap().get("selectedPluginBean");
		if (selectedPluginBean != null && selectedPluginBean.getPlugin() == plugin) {
			selectedPluginBean.setPlugin((MIPlugin) null);
		}
	}

	/**
	 * This method extracts the connections between the filters from the current main project.
	 * 
	 * @return A list with all connections available.
	 */
	private List<Connection> getConnectionsFromProject() {
		final List<Connection> result = new ArrayList<Connection>();

		if (this.mainProject != null) {
			final EList<MIPlugin> mPlugins = this.mainProject.getPlugins();
			for (final MIPlugin mPlugin : mPlugins) {
				final EList<MIOutputPort> mOutputPorts = mPlugin.getOutputPorts();
				for (final MIOutputPort mOutputPort : mOutputPorts) {
					final EList<MIInputPort> mInputPorts = mOutputPort.getSubscribers();
					for (final MIInputPort mInputPort : mInputPorts) {
						result.add(new Connection(mPlugin, mInputPort.getParent(), mInputPort, mOutputPort));
					}
				}
			}
		}

		return result;
	}

	/**
	 * Delivers the connections (between the filters) within the current main project.
	 * 
	 * @return A list containing all available connections.
	 */
	public List<Connection> getConnections() {
		return this.connections;
	}

	/**
	 * Delivers the valid connections (between the filters) within the current main project.
	 * 
	 * @return A list containing all available and valid connections.
	 */
	public List<Connection> getValidConnections() {
		final List<Connection> validConnections = new ArrayList<Connection>();
		final List<Connection> availableConnections = this.getConnections();

		for (final Connection connection : availableConnections) {
			if (connection.isValid()) {
				validConnections.add(connection);
			}
		}

		return validConnections;
	}

	/**
	 * This method adds an empty connection to the current main project.
	 */
	public void addConnection() {
		this.connections.add(new Connection(null, null, null, null));
	}

	/**
	 * This method "submits" the current connections to the main project (In other words: The connections will be stored within the main project).
	 */
	public void submitConnections() {
		for (final Connection connection : this.connections) {
			if (connection.isValid()) {
				connection.getOutputPort().getSubscribers().add(connection.getInputPort());
			}
		}
	}

	/**
	 * This method returns all available filters of the current main project. In other words: It returns the same as mainProject.getPlugin() but only those which are
	 * really instances of {@link MIFilter}.
	 * 
	 * @return A list with all filters of the main project.
	 */
	public List<MIFilter> getFilters() {
		final List<MIFilter> result = new ArrayList<MIFilter>();
		final List<MIPlugin> plugins = this.mainProject.getPlugins();

		for (final MIPlugin plugin : plugins) {
			if (plugin instanceof MIFilter) {
				result.add((MIFilter) plugin);
			}
		}

		return result;
	}

	/**
	 * This method is used to initialize the object after creation.
	 */
	@PostConstruct
	@SuppressWarnings("FinalPrivateMethod")
	private final void postConstruct() {
		// Register this instance as an observer
		this.observableBean.addObserver(this);
	}

	/**
	 * This method is used to finalize the object before destruction.
	 */
	@PreDestroy
	@SuppressWarnings("FinalPrivateMethod")
	private final void preDestroy() {
		// Unregister this instance as an observer
		this.observableBean.deleteObserver(this);
	}

	@Override
	public void update(final Observable o, final Object arg) {
		// If the current main project has just been removed, we have to set the current main project to null.
		if (arg instanceof ProjectsBean.MsgProjectRemoved && ((ProjectsBean.MsgProjectRemoved) arg).getProject() == this.mainProject) {
			this.setMainProject(null);
		}
	}

	/**
	 * This helper class is a wrapper for a project and can be send to the observers as a message that the main project selection just has been changed. It contains
	 * the newly selected main project.
	 * 
	 * @author Nils Christian Ehmke
	 */
	public static final class MsgMainProjectSelection {
		/**
		 * The stored project.
		 */
		private final MIProject project;

		/**
		 * Creates a new instance of this class using the given parameters.
		 * 
		 * @param project
		 *            The project to be stored within this class.
		 */
		public MsgMainProjectSelection(final MIProject project) {
			this.project = project;
		}

		/**
		 * Delivers the stored project within this class.
		 * 
		 * @return The instance of {@link MIProject}.
		 */
		public MIProject getProject() {
			return this.project;
		}
	}
}
