/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.beans.session;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.context.FacesContext;

import kieker.analysis.model.analysisMetaModel.MIPlugin;

import org.primefaces.context.RequestContext;

/**
 * This bean can be used to save the currently selected plugin of the user. This bean registers to the user's instance of {@link SelectedMainProjectBean} as an
 * observer. If the main project changes, the content of this bean changes as well. This bean also updates the properties form if the selection of the plugin
 * changes.
 * 
 * @author Nils Christian Ehmke
 * @version 1.0
 */
@ManagedBean
@SessionScoped
public final class SelectedPluginBean implements Observer {

	/**
	 * This is the name of the target element to be updated when the plugin selection changes.
	 */
	private static final String UPDATE_TARGET = "propertiesForm";

	/**
	 * This is the main project bean where this instance registers as an observer.
	 */
	private final SelectedMainProjectBean observableBean;

	/**
	 * The plugin which is stored by this container.
	 */
	private MIPlugin plugin;

	/**
	 * Creates a new instance of this class.
	 */
	public SelectedPluginBean() {
		// Try to get the main project bean.
		final FacesContext context = FacesContext.getCurrentInstance();
		this.observableBean = context.getApplication().evaluateExpressionGet(context, "#{selectedMainProjectBean}", SelectedMainProjectBean.class);
	}

	/**
	 * This method is used to initialize the object after creation.
	 */
	@PostConstruct
	@SuppressWarnings("FinalPrivateMethod")
	private final void postConstruct() {
		// Register this instance as an observer
		this.observableBean.addObserver(this);
	}

	/**
	 * This method is used to finalize the object before destruction.
	 */
	@PreDestroy
	@SuppressWarnings("FinalPrivateMethod")
	private final void preDestroy() {
		// Unregister this instance as an observer
		this.observableBean.deleteObserver(this);
	}

	/**
	 * Delivers the stored plugin.
	 * 
	 * @return The plugin within this container.
	 */
	public MIPlugin getPlugin() {
		return this.plugin;
	}

	/**
	 * Sets the plugin stored by this bean and updates (if possible) the corresponding element in the current context.
	 * 
	 * @param plugin
	 *            The new bean to be stored within this bean.
	 */
	public void setPlugin(final MIPlugin plugin) {
		this.plugin = plugin;

		// Update the corresponding components
		final RequestContext requestContext = RequestContext.getCurrentInstance();
		requestContext.addPartialUpdateTarget(SelectedPluginBean.UPDATE_TARGET);
	}

	/**
	 * This method delivers a list containing all properties of the current plugin, but adds the name as a string to it.
	 * 
	 * @return A list with all properties plus the name (as a string). If no plugin is selected, this method returns an empty list, never null.
	 */
	public List<Object> getAdvancedProperties() {
		final List<Object> resultList = new ArrayList<Object>();

		if (this.plugin != null) {
			resultList.add(this.plugin.getName());
			resultList.addAll(this.plugin.getProperties());
		}

		return resultList;
	}

	@Override
	public void update(final Observable o, final Object arg) {
		// If the main project has been newly selected, we have to set the current plugin to null
		if (arg instanceof SelectedMainProjectBean.MsgMainProjectSelection) {
			this.setPlugin(null);
		}
	}
}
