/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/
package kieker.webgui.beans.session;

import java.util.Observable;
import java.util.Observer;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.context.FacesContext;

import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.webgui.beans.application.ProjectsBean;

import org.primefaces.event.NodeSelectEvent;
import org.primefaces.model.TreeNode;

/**
 * This bean can be used to store the currently selected project and the current main project for the user within a session.
 * 
 * @author Nils Christian Ehmke
 * @version 1.0
 */
@ManagedBean
@SessionScoped
public class SelectedProjectBean implements Observer {

	/**
	 * The currently selected node of the current user.
	 */
	private TreeNode selectedNode;
	/**
	 * The selected project of the current user.
	 */
	private MIProject selectedProject;

	private final ProjectsBean observableBean;

	/**
	 * Creates a new instance of this class.
	 */
	public SelectedProjectBean() {
		// Try to get the projects bean.
		final FacesContext context = FacesContext.getCurrentInstance();
		this.observableBean = context.getApplication().evaluateExpressionGet(context, "#{projectsBean}", ProjectsBean.class);
	}

	/**
	 * Delivers the project currently selected by the user. It can be null.
	 * 
	 * @return The user's currently selected project.
	 */
	public final MIProject getSelectedProject() {
		return this.selectedProject;
	}

	/**
	 * Sets the currently selected project within the bean.
	 * 
	 * @param selectedProject
	 *            The selected project.
	 */
	public final void setSelectedProject(final MIProject selectedProject) {
		this.selectedProject = selectedProject;
	}

	/**
	 * Delivers the currently selected node.
	 * 
	 * @return The selected node.
	 */
	public final TreeNode getSelectedNode() {
		return this.selectedNode;
	}

	/**
	 * This event should be triggered once a new node is selected to make sure that the newly selected node is stored within this bean.
	 * 
	 * @param event
	 *            The event that a node has been selected by the user.
	 */
	public final void onNodeSelect(final NodeSelectEvent event) {
		this.setSelectedNode(event.getTreeNode());
	}

	/**
	 * Sets the currently selected node to a new value.
	 * 
	 * @param selectedNode
	 *            The selected node.
	 */
	public final void setSelectedNode(final TreeNode selectedNode) {
		this.selectedNode = selectedNode;
		if (selectedNode != null && selectedNode.getData() instanceof MIProject) {
			this.setSelectedProject((MIProject) selectedNode.getData());
		} else {
			this.setSelectedProject(null);
		}
	}

	/**
	 * This method is used to initialize the object after creation.
	 */
	@PostConstruct
	@SuppressWarnings("FinalPrivateMethod")
	private final void postConstruct() {
		// Register this instance as an observer
		this.observableBean.addObserver(this);
	}

	/**
	 * This method is used to finalize the object before destruction.
	 */
	@PreDestroy
	@SuppressWarnings("FinalPrivateMethod")
	private final void preDestroy() {
		// Unregister this instance as an observer
		this.observableBean.deleteObserver(this);
	}

	@Override
	public void update(final Observable o, final Object arg) {
		// If the current main project has just been removed, we have to set the current main project to null.
		if (arg instanceof ProjectsBean.MsgProjectRemoved && ((ProjectsBean.MsgProjectRemoved) arg).getProject() == this.selectedProject) {
			this.setSelectedProject(null);
		}
	}
}
