/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.common;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map.Entry;

/**
 * This is a thread-safe cache, which can be used to cache objects under a specific key. This class is a LRU cache.
 * 
 * @author Nils Christian Ehmke
 * 
 * @param <Key>
 *            The type for the keys.
 * @param <Value>
 *            The type for the values.
 */
public class LRUCache<Key, Value> {

	/**
	 * This is the map which is used to store the entries.
	 */
	private final HashMap<Key, Value> hashMap;
	private final int maxSize;

	/**
	 * Creates a new instance of this class using the given size as maximal cache size.
	 * 
	 * @param size
	 *            The maximal cache size.
	 */
	public LRUCache(final int size) {
		this.maxSize = size;
		this.hashMap = new FixedLinkedHashMap();
	}

	/**
	 * This method adds a given value within the cache, using the given key. If a value with the same key does already exist, it will be replaced. If the entry would
	 * excess the maximal cache size, the oldest entry will be removed.
	 * 
	 * @param key
	 *            The key used to store the value.
	 * @param value
	 *            The value to be stored.
	 */
	public void add(final Key key, final Value value) {
		synchronized (this.hashMap) {
			this.hashMap.put(key, value);
		}
	}

	/**
	 * This method delivers the value to the key or null, if the value doesn't exist.
	 * 
	 * @param key
	 *            The key to the value.
	 * @return The corresponding value.
	 */
	public Value get(final Key key) {
		synchronized (this.hashMap) {
			return this.hashMap.get(key);
		}
	}

	/**
	 * A modified version of {@link LinkedHashMap} which has a fixed size. Each time the {@code put} or {@code putAll} method exceeds the size of the
	 * {@link LRUCache}, the oldest entry is being removed.
	 * 
	 * @author Nils Christian Ehmke
	 */
	private class FixedLinkedHashMap extends LinkedHashMap<Key, Value> {

		/**
		 * The serial version uid.
		 */
		private static final long serialVersionUID = 1L;

		/**
		 * Creates a new instance of this class, using the LRUCache's maximal size as the size of the map.
		 */
		public FixedLinkedHashMap() {
			super(LRUCache.this.maxSize);
		}

		@Override
		protected boolean removeEldestEntry(final Entry<Key, Value> eldest) {
			return this.size() > LRUCache.this.maxSize;
		}

	}
}
