/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/
package kieker.webgui.common;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import kieker.analysis.plugin.AbstractPlugin;
import kieker.analysis.plugin.annotation.Plugin;
import kieker.analysis.repository.AbstractRepository;

/**
 * This tool class can be used to find all plugins and repositories within a given jar file - assuming that the <code>PluginClassLoader</code> knows these jars.
 * 
 * @author Nils Christian Ehmke
 * @version 1.0
 *          TODO Cache and observer update!
 */
public final class PluginFinder {

	private static final LRUCache<URL, List<Class<AbstractPlugin>>> PLUGIN_CACHE;
	private static final LRUCache<URL, List<Class<AbstractRepository>>> REPOSITORY_CACHE;

	static {
		PLUGIN_CACHE = new LRUCache<URL, List<Class<AbstractPlugin>>>(15);
		REPOSITORY_CACHE = new LRUCache<URL, List<Class<AbstractRepository>>>(15);
	}

	/**
	 * Creates a new instance of this class.
	 */
	private PluginFinder() {
		// No code necessary.
	}

	/**
	 * This method delivers all classes which are available in the given jar and are compatible with <code>AbstractRepository</code>).
	 * 
	 * @param url
	 *            The url for the jar.
	 * @return A list containing all available repository-classes or null, if an exception occurred.
	 */
	public static List<Class<AbstractRepository>> getAllRepositoriesWithinJar(final URL url) {
		// TODO: Merge this with the other method
		final List<Class<AbstractRepository>> repositoryClasses = PluginFinder.REPOSITORY_CACHE.get(url);
		if (repositoryClasses != null) {
			return repositoryClasses;
		}
		try {
			// Open the jar file and run through all entries within this file.
			final JarFile jarFile = new JarFile(new File(url.getPath()));
			final List<Class<AbstractRepository>> result = new ArrayList<Class<AbstractRepository>>();
			final Enumeration<JarEntry> jarEntries = jarFile.entries();
			while (jarEntries.hasMoreElements()) {
				final JarEntry jarEntry = jarEntries.nextElement();
				try {
					String name = jarEntry.toString();
					name = name.replace('/', '.');
					name = name.replace(".class", "");
					// Try to find a class with the same name.
					final Class<?> c = PluginClassLoader.getInstance().loadClass(name);

					if (AbstractRepository.class.isAssignableFrom(c)) {
						result.add((Class<AbstractRepository>) c);
					}
				} catch (final Throwable ex) { // NOCS (IllegalCatchCheck)
					// Ignore error.
				}
			}
			jarFile.close();
			PluginFinder.REPOSITORY_CACHE.add(url, result);
			return result;
		} catch (final IOException ex) {
			ex.printStackTrace();
		}
		return null;
	}

	/**
	 * This method delivers all classes which are available in the given jar and have the Plugin-Annotation from the Kieker framework (And are correctly compatible
	 * with <code>AbstractPlugin</code>).
	 * 
	 * @param url
	 *            The url for the jar.
	 * @return A list containing all available plugin-classes or null, if an exception occurred.
	 */
	public static List<Class<AbstractPlugin>> getAllPluginsWithinJar(final URL url) {
		final List<Class<AbstractPlugin>> pluginClasses = PluginFinder.PLUGIN_CACHE.get(url);
		if (pluginClasses != null) {
			return pluginClasses;
		}
		try {
			// Open the jar file and run through all entries within this file.
			final JarFile jarFile = new JarFile(new File(url.getPath()));
			final List<Class<AbstractPlugin>> result = new ArrayList<Class<AbstractPlugin>>();
			final Enumeration<JarEntry> jarEntries = jarFile.entries();
			while (jarEntries.hasMoreElements()) {
				final JarEntry jarEntry = jarEntries.nextElement();
				try {
					String name = jarEntry.toString();
					name = name.replace('/', '.');
					name = name.replace(".class", "");
					// Try to find a class with the same name.
					final Class<?> c = PluginClassLoader.getInstance().loadClass(name);
					// If it is a class and has the annotation - put it into our list.
					if (c.isAnnotationPresent(Plugin.class) && AbstractPlugin.class.isAssignableFrom(c)) {
						result.add((Class<AbstractPlugin>) c);
					}
				} catch (final Throwable ex) { // NOCS (IllegalCatchCheck)
					// Ignore error.
				}
			}
			jarFile.close();
			PluginFinder.PLUGIN_CACHE.add(url, result);
			return result;
		} catch (final IOException ex) {
			ex.printStackTrace();
		}
		return null;
	}
}
